#include <Ice/Ice.h>
#include <MWSessionI.h>
#include <list>
#include <MWMap.h>

using namespace std;
using namespace MW;


//
// MWSessionI
//

MWSessionI::MWSessionI(const string& userId) :
    _userId(userId)
{
}

void
MWSessionI::setCallback(shared_ptr<MWCallbackPrx> callback, const Ice::Current& current)
{
  
  lock_guard<mutex> sync(mutex);
  if(!_callback)
    {
      _callback = callback;
      auto mwMap = MWMap::instance();
      // mwMap->message(_userId + " has enter the map");
      mwMap->enter(Ice::uncheckedCast<MWSessionPrx>(current.adapter->createProxy(current.id)), callback, current);
    }
}

void
MWSessionI::sendPosition(string data, const Ice::Current&)
{
  lock_guard<mutex> sync(_mutex);
  MWMap::instance()->message(_userId + " " + data);
}

void
MWSessionI::destroy(const Ice::Current& current)
{
  cout << "Destrying session for " << _userId << endl;
  lock_guard<mutex> sync(_mutex);
  if (_callback)
    {
      
      auto mwMap = MWMap::instance();
      mwMap->leave(_callback, current);
      _callback = nullptr;
      mwMap->message(_userId + " left the Map");
      
    }
  current.adapter->remove(current.id);
}

//
// MWSessionManagerI
//

shared_ptr<Glacier2::SessionPrx>
MWSessionManagerI::create(string userId, shared_ptr<Glacier2::SessionControlPrx>, const Ice::Current& current)
{
  Ice::Identity ident = { Ice::generateUUID(), "sesion"};
  return Ice::uncheckedCast<Glacier2::SessionPrx>(current.adapter->add(make_shared<MWSessionI>(userId), ident));
}

void
MWSessionManagerI::destroy()
{
  MWMap::instance()->destroy();
}
